@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "config.h"
@@ -46,12 +46,11 @@ enum {
 
 struct _DjvuDocumentClass
 {
-	GObjectClass parent_class;
+	EvDocumentClass parent_class;
 };
 
 typedef struct _DjvuDocumentClass DjvuDocumentClass;
 
-static void djvu_document_document_iface_init (EvDocumentIface *iface);
 static void djvu_document_document_thumbnails_iface_init (EvDocumentThumbnailsIface *iface);
 static void djvu_document_file_exporter_iface_init (EvFileExporterIface *iface);
 static void djvu_document_find_iface_init (EvDocumentFindIface *iface);
@@ -75,7 +74,7 @@ ev_djvu_error_quark (void)
 {
 	static GQuark q = 0;
 	if (q == 0)
-		q = g_quark_from_static_string ("ev-djvu-quark");
+		q = g_quark_from_string ("ev-djvu-quark");
 	
 	return q;
 }
@@ -171,7 +170,7 @@ djvu_document_load (EvDocument  *document,
     		g_set_error_literal (error,
 				     EV_DOCUMENT_ERROR,
 				     EV_DOCUMENT_ERROR_INVALID,
-				     _("DJVU document has incorrect format"));
+				     _("DjVu document has incorrect format"));
 		return FALSE;
 	}
 
@@ -249,7 +248,7 @@ djvu_document_load (EvDocument  *document,
                                      G_FILE_ERROR,
                                      G_FILE_ERROR_EXIST,
 				     _("The document is composed of several files. "
-                                       "One or more of such files cannot be accessed."));
+                                       "One or more of these files cannot be accessed."));
 
 		return FALSE;
 	}
@@ -324,7 +323,6 @@ djvu_document_render (EvDocument      *document,
 	ddjvu_page_t *d_page;
 	ddjvu_page_rotation_t rotation;
 	double page_width, page_height, tmp;
-	static const cairo_user_data_key_t key;
 
 	d_page = ddjvu_page_create_by_pageno (djvu_document->d_document, rc->page->index);
 	
@@ -356,19 +354,12 @@ djvu_document_render (EvDocument      *document,
 	        default:
 			rotation = DDJVU_ROTATE_0;
 	}
-#ifdef HAVE_CAIRO_FORMAT_STRIDE_FOR_WIDTH
-	rowstride = cairo_format_stride_for_width (CAIRO_FORMAT_RGB24, page_width);
-#else
-	rowstride = page_width * 4;
-#endif
-	pixels = (gchar *) g_malloc (page_height * rowstride);
-	surface = cairo_image_surface_create_for_data ((guchar *)pixels,
-						       CAIRO_FORMAT_RGB24,
-						       page_width,
-						       page_height,
-						       rowstride);
-	cairo_surface_set_user_data (surface, &key,
-				     pixels, (cairo_destroy_func_t)g_free);
+
+	surface = cairo_image_surface_create (CAIRO_FORMAT_RGB24,
+					      page_width, page_height);
+	rowstride = cairo_image_surface_get_stride (surface);
+	pixels = (gchar *)cairo_image_surface_get_data (surface);
+
 	prect.x = 0;
 	prect.y = 0;
 	prect.w = page_width;
@@ -384,6 +375,8 @@ djvu_document_render (EvDocument      *document,
 			   rowstride,
 			   pixels);
 
+	cairo_surface_mark_dirty (surface);
+
 	return surface;
 }
 
@@ -412,30 +405,16 @@ djvu_document_finalize (GObject *object)
 static void
 djvu_document_class_init (DjvuDocumentClass *klass)
 {
-	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+	GObjectClass    *gobject_class = G_OBJECT_CLASS (klass);
+	EvDocumentClass *ev_document_class = EV_DOCUMENT_CLASS (klass);
 
 	gobject_class->finalize = djvu_document_finalize;
-}
 
-static EvDocumentInfo *
-djvu_document_get_info (EvDocument *document)
-{
-	EvDocumentInfo *info;
-
-	info = g_new0 (EvDocumentInfo, 1);
-
-	return info;
-}
-
-static void
-djvu_document_document_iface_init (EvDocumentIface *iface)
-{
-	iface->load = djvu_document_load;
-	iface->save = djvu_document_save;
-	iface->get_n_pages = djvu_document_get_n_pages;
-	iface->get_page_size = djvu_document_get_page_size;
-	iface->render = djvu_document_render;
-	iface->get_info = djvu_document_get_info;
+	ev_document_class->load = djvu_document_load;
+	ev_document_class->save = djvu_document_save;
+	ev_document_class->get_n_pages = djvu_document_get_n_pages;
+	ev_document_class->get_page_size = djvu_document_get_page_size;
+	ev_document_class->render = djvu_document_render;
 }
 
 static gchar *
@@ -712,9 +691,9 @@ djvu_document_find_iface_init (EvDocumentFindIface *iface)
 
 static GList *
 djvu_document_links_get_links (EvDocumentLinks *document_links,
-			       gint             page)
+			       EvPage          *page)
 {
-	return djvu_links_get_links (document_links, page, SCALE_FACTOR);
+	return djvu_links_get_links (document_links, page->index, SCALE_FACTOR);
 }
 
 static void
