@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include <config.h>
@@ -31,17 +31,16 @@
 #include "ev-document-misc.h"
 
 struct _PSDocument {
-	GObject object;
+	EvDocument object;
 
 	SpectreDocument *doc;
 	SpectreExporter *exporter;
 };
 
 struct _PSDocumentClass {
-	GObjectClass parent_class;
+	EvDocumentClass parent_class;
 };
 
-static void ps_document_document_iface_init            (EvDocumentIface           *iface);
 static void ps_document_file_exporter_iface_init       (EvFileExporterIface       *iface);
 static void ps_document_document_thumbnails_iface_init (EvDocumentThumbnailsIface *iface);
 
@@ -77,16 +76,6 @@ ps_document_dispose (GObject *object)
 	G_OBJECT_CLASS (ps_document_parent_class)->dispose (object);
 }
 
-static void
-ps_document_class_init (PSDocumentClass *klass)
-{
-	GObjectClass *object_class;
-
-	object_class = G_OBJECT_CLASS (klass);
-
-	object_class->dispose = ps_document_dispose;
-}
-
 /* EvDocumentIface */
 static gboolean
 ps_document_load (EvDocument *document,
@@ -270,6 +259,16 @@ ps_document_get_info (EvDocument *document)
 	return info;
 }
 
+static gboolean
+ps_document_get_backend_info (EvDocument            *document,
+			      EvDocumentBackendInfo *info)
+{
+	info->name = "libspectre";
+	info->version = SPECTRE_VERSION_STRING;
+
+	return TRUE;
+}
+
 static cairo_surface_t *
 ps_document_render (EvDocument      *document,
 		    EvRenderContext *rc)
@@ -331,16 +330,22 @@ ps_document_render (EvDocument      *document,
 }
 
 static void
-ps_document_document_iface_init (EvDocumentIface *iface)
+ps_document_class_init (PSDocumentClass *klass)
 {
-	iface->load = ps_document_load;
-	iface->save = ps_document_save;
-	iface->get_n_pages = ps_document_get_n_pages;
-	iface->get_page = ps_document_get_page;
-	iface->get_page_size = ps_document_get_page_size;
-	iface->get_page_label = ps_document_get_page_label;
-	iface->get_info = ps_document_get_info;
-	iface->render = ps_document_render;
+	GObjectClass    *object_class = G_OBJECT_CLASS (klass);
+	EvDocumentClass *ev_document_class = EV_DOCUMENT_CLASS (klass);
+
+	object_class->dispose = ps_document_dispose;
+
+	ev_document_class->load = ps_document_load;
+	ev_document_class->save = ps_document_save;
+	ev_document_class->get_n_pages = ps_document_get_n_pages;
+	ev_document_class->get_page = ps_document_get_page;
+	ev_document_class->get_page_size = ps_document_get_page_size;
+	ev_document_class->get_page_label = ps_document_get_page_label;
+	ev_document_class->get_info = ps_document_get_info;
+	ev_document_class->get_backend_info = ps_document_get_backend_info;
+	ev_document_class->render = ps_document_render;
 }
 
 /* EvDocumentThumbnailsIface */
