@@ -14,33 +14,19 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  */
 
 #include <config.h>
 #include "ev-document-thumbnails.h"
+#include "ev-document.h"
 
-GType
-ev_document_thumbnails_get_type (void)
+EV_DEFINE_INTERFACE (EvDocumentThumbnails, ev_document_thumbnails, 0)
+
+static void
+ev_document_thumbnails_class_init (EvDocumentThumbnailsIface *klass)
 {
-	static GType type = 0;
-
-	if (G_UNLIKELY (type == 0))
-	{
-		const GTypeInfo our_info =
-		{
-			sizeof (EvDocumentThumbnailsIface),
-			NULL,
-			NULL,
-		};
-
-		type = g_type_register_static (G_TYPE_INTERFACE,
-					       "EvDocumentThumbnails",
-					       &our_info, (GTypeFlags)0);
-	}
-
-	return type;
 }
 
 GdkPixbuf *
