@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include <config.h>
@@ -35,7 +35,7 @@ typedef struct {
 /* Evince stock icons */
 static const EvStockIcon stock_icons [] = {
 	{ EV_STOCK_ZOOM,	     "zoom" },
-	{ EV_STOCK_ZOOM_PAGE,        "zoom-fit-page" },
+	{ EV_STOCK_ZOOM_PAGE,        "zoom-fit-height" },
 	{ EV_STOCK_ZOOM_WIDTH,       "zoom-fit-width" },
 	{ EV_STOCK_VIEW_DUAL,        "view-page-facing" },
 	{ EV_STOCK_VIEW_CONTINUOUS,  "view-page-continuous" },
@@ -45,7 +45,8 @@ static const EvStockIcon stock_icons [] = {
 	{ EV_STOCK_VISIBLE,          "eye"},
 	{ EV_STOCK_RESIZE_SE,        "resize-se"},
 	{ EV_STOCK_RESIZE_SW,        "resize-sw"},
-	{ EV_STOCK_CLOSE,            "close"}
+	{ EV_STOCK_CLOSE,            "close"},
+	{ EV_STOCK_INVERTED_COLORS,  "stock_filters-invert"}
 };
 
 static gchar *ev_icons_path;
