@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -26,10 +26,10 @@
 #include <gtk/gtk.h>
 
 #include "ev-document-fonts.h"
-#include "ev-page-cache.h"
 #include "ev-properties-dialog.h"
 #include "ev-properties-fonts.h"
 #include "ev-properties-view.h"
+#include "ev-properties-license.h"
 
 struct _EvPropertiesDialog {
 	GtkDialog base_instance;
@@ -38,6 +38,7 @@ struct _EvPropertiesDialog {
 	GtkWidget *notebook;
 	GtkWidget *general_page;
 	GtkWidget *fonts_page;
+	GtkWidget *license_page;
 };
 
 struct _EvPropertiesDialogClass {
@@ -85,7 +86,7 @@ ev_properties_dialog_set_document (EvPropertiesDialog *properties,
 
 	properties->document = document;
 
-	info = ev_page_cache_get_info (ev_page_cache_get (document));
+	info = ev_document_get_info (document);
 
 	if (properties->general_page == NULL) {
 		label = gtk_label_new (_("General"));
@@ -108,6 +109,19 @@ ev_properties_dialog_set_document (EvPropertiesDialog *properties,
 		ev_properties_fonts_set_document
 			(EV_PROPERTIES_FONTS (properties->fonts_page), document);
 	}
+
+	if (info->fields_mask & EV_DOCUMENT_INFO_LICENSE && info->license) {
+		if (properties->license_page == NULL) {
+			label = gtk_label_new (_("Document License"));
+			properties->license_page = ev_properties_license_new ();
+			gtk_notebook_append_page (GTK_NOTEBOOK (properties->notebook),
+						  properties->license_page, label);
+			gtk_widget_show (properties->license_page);
+		}
+
+		ev_properties_license_set_license
+			(EV_PROPERTIES_LICENSE (properties->license_page), info->license);
+	}
 }
 
 GtkWidget *
