@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -45,6 +45,18 @@
  * limit its use */
 #define MAX_ICON_VIEW_PAGE_COUNT 1500
 
+typedef struct _EvThumbsSize
+{
+	gint width;
+	gint height;
+} EvThumbsSize;
+
+typedef struct _EvThumbsSizeCache {
+	gboolean uniform;
+	gint uniform_width;
+	gint uniform_height;
+	EvThumbsSize *sizes;
+} EvThumbsSizeCache;
 
 struct _EvSidebarThumbnailsPrivate {
 	GtkWidget *swindow;
@@ -54,11 +66,13 @@ struct _EvSidebarThumbnailsPrivate {
 	GtkListStore *list_store;
 	GHashTable *loading_icons;
 	EvDocument *document;
-	EvPageCache *page_cache;
+	EvDocumentModel *model;
+	EvThumbsSizeCache *size_cache;
 
 	gint n_pages, pages_done;
 
 	int rotation;
+	gboolean inverted_colors;
 
 	/* Visible pages */
 	gint start_page, end_page;
@@ -81,8 +95,6 @@ static void         ev_sidebar_thumbnails_clear_model      (EvSidebarThumbnails
 static gboolean     ev_sidebar_thumbnails_support_document (EvSidebarPage       *sidebar_page,
 							    EvDocument          *document);
 static void         ev_sidebar_thumbnails_page_iface_init  (EvSidebarPageIface  *iface);
-static void         ev_sidebar_thumbnails_set_document     (EvSidebarPage       *sidebar_page,
-							    EvDocument          *document);
 static const gchar* ev_sidebar_thumbnails_get_label        (EvSidebarPage       *sidebar_page);
 static void         thumbnail_job_completed_callback       (EvJobThumbnail      *job,
 							    EvSidebarThumbnails *sidebar_thumbnails);
@@ -98,6 +110,138 @@ G_DEFINE_TYPE_EXTENDED (EvSidebarThumbnails,
 #define EV_SIDEBAR_THUMBNAILS_GET_PRIVATE(object) \
 	(G_TYPE_INSTANCE_GET_PRIVATE ((object), EV_TYPE_SIDEBAR_THUMBNAILS, EvSidebarThumbnailsPrivate));
 
+/* Thumbnails dimensions cache */
+#define EV_THUMBNAILS_SIZE_CACHE_KEY "ev-thumbnails-size-cache"
+
+static EvThumbsSizeCache *
+ev_thumbnails_size_cache_new (EvDocument *document)
+{
+	EvThumbsSizeCache *cache;
+	EvRenderContext *rc = NULL;
+	gint i, n_pages;
+	EvThumbsSize *thumb_size;
+
+	cache = g_new0 (EvThumbsSizeCache, 1);
+
+	n_pages = ev_document_get_n_pages (document);
+
+	/* Assume all pages are the same size until proven otherwise */
+	cache->uniform = TRUE;
+
+	for (i = 0; i < n_pages; i++) {
+		EvPage *page;
+		gdouble page_width, page_height;
+		gint    thumb_width = 0;
+		gint    thumb_height = 0;
+
+		page = ev_document_get_page (document, i);
+
+		ev_document_get_page_size (document, i, &page_width, &page_height);
+
+		if (!rc) {
+			rc = ev_render_context_new (page, 0, (gdouble)THUMBNAIL_WIDTH / page_width);
+		} else {
+			ev_render_context_set_page (rc, page);
+			ev_render_context_set_scale (rc, (gdouble)THUMBNAIL_WIDTH / page_width);
+		}
+
+		ev_document_thumbnails_get_dimensions (EV_DOCUMENT_THUMBNAILS (document),
+						       rc, &thumb_width, &thumb_height);
+
+		if (i == 0) {
+			cache->uniform_width = thumb_width;
+			cache->uniform_height = thumb_height;
+		} else if (cache->uniform &&
+			   (cache->uniform_width != thumb_width ||
+			    cache->uniform_height != thumb_height)) {
+			/* It's a different thumbnail size.  Backfill the array. */
+			int j;
+
+			cache->sizes = g_new0 (EvThumbsSize, n_pages);
+
+			for (j = 0; j < i; j++) {
+				thumb_size = &(cache->sizes[j]);
+				thumb_size->width = cache->uniform_width;
+				thumb_size->height = cache->uniform_height;
+			}
+			cache->uniform = FALSE;
+		}
+
+		if (! cache->uniform) {
+			thumb_size = &(cache->sizes[i]);
+
+			thumb_size->width = thumb_width;
+			thumb_size->height = thumb_height;
+		}
+
+		g_object_unref (page);
+	}
+
+	if (rc) {
+		g_object_unref (rc);
+	}
+
+	return cache;
+}
+
+static void
+ev_thumbnails_size_cache_get_size (EvThumbsSizeCache *cache,
+				   gint               page,
+				   gint               rotation,
+				   gint              *width,
+				   gint              *height)
+{
+	gint w, h;
+
+	if (cache->uniform) {
+		w = cache->uniform_width;
+		h = cache->uniform_height;
+	} else {
+		EvThumbsSize *thumb_size;
+
+		thumb_size = &(cache->sizes[page]);
+
+		w = thumb_size->width;
+		h = thumb_size->height;
+	}
+
+	if (rotation == 0 || rotation == 180) {
+		if (width) *width = w;
+		if (height) *height = h;
+	} else {
+		if (width) *width = h;
+		if (height) *height = w;
+	}
+}
+
+static void
+ev_thumbnails_size_cache_free (EvThumbsSizeCache *cache)
+{
+	if (cache->sizes) {
+		g_free (cache->sizes);
+		cache->sizes = NULL;
+	}
+
+	g_free (cache);
+}
+
+static EvThumbsSizeCache *
+ev_thumbnails_size_cache_get (EvDocument *document)
+{
+	EvThumbsSizeCache *cache;
+
+	cache = g_object_get_data (G_OBJECT (document), EV_THUMBNAILS_SIZE_CACHE_KEY);
+	if (!cache) {
+		cache = ev_thumbnails_size_cache_new (document);
+		g_object_set_data_full (G_OBJECT (document),
+					EV_THUMBNAILS_SIZE_CACHE_KEY,
+					cache,
+					(GDestroyNotify)ev_thumbnails_size_cache_free);
+	}
+
+	return cache;
+}
+
 
 static void
 ev_sidebar_thumbnails_dispose (GObject *object)
@@ -114,7 +258,7 @@ ev_sidebar_thumbnails_dispose (GObject *object)
 		g_object_unref (sidebar_thumbnails->priv->list_store);
 		sidebar_thumbnails->priv->list_store = NULL;
 	}
-	
+
 	G_OBJECT_CLASS (ev_sidebar_thumbnails_parent_class)->dispose (object);
 }
 
@@ -237,7 +381,7 @@ clear_range (EvSidebarThumbnails *sidebar_thumbnails,
 			g_object_unref (job);
 		}
 
-		ev_page_cache_get_thumbnail_size (priv->page_cache, start_page,
+		ev_thumbnails_size_cache_get_size (priv->size_cache, start_page,
 						  priv->rotation,
 						  &width, &height);
 		if (!loading_icon || (width != prev_width && height != prev_height)) {
@@ -263,13 +407,11 @@ get_scale_for_page (EvSidebarThumbnails *sidebar_thumbnails,
 		    gint                 page)
 {
 	EvSidebarThumbnailsPrivate *priv = sidebar_thumbnails->priv;
-	gint width, height;
+	gdouble width;
 
-	ev_page_cache_get_size (priv->page_cache,
-				page, 0,
-				1.0, &width, &height);
-	
-	return (gdouble)THUMBNAIL_WIDTH / (gdouble)width;
+	ev_document_get_page_size (priv->document, page, &width, NULL);
+
+	return (gdouble)THUMBNAIL_WIDTH / width;
 }
 
 static void
@@ -417,9 +559,9 @@ ev_sidebar_thumbnails_fill_model (EvSidebarThumbnails *sidebar_thumbnails)
 		GdkPixbuf *loading_icon = NULL;
 		gint       width, height;
 
-		page_label = ev_page_cache_get_page_label (priv->page_cache, i);
+		page_label = ev_document_get_page_label (priv->document, i);
 		page_string = g_markup_printf_escaped ("<i>%s</i>", page_label);
-		ev_page_cache_get_thumbnail_size (sidebar_thumbnails->priv->page_cache, i,
+		ev_thumbnails_size_cache_get_size (sidebar_thumbnails->priv->size_cache, i,
 						  sidebar_thumbnails->priv->rotation,
 						  &width, &height);
 		if (!loading_icon || (width != prev_width && height != prev_height)) {
@@ -442,34 +584,6 @@ ev_sidebar_thumbnails_fill_model (EvSidebarThumbnails *sidebar_thumbnails)
 	}
 }
 
-static gboolean
-refresh (EvSidebarThumbnails *sidebar_thumbnails)
-{
-	adjustment_changed_cb (sidebar_thumbnails);
-	return FALSE;
-}
-
-void
-ev_sidebar_thumbnails_refresh (EvSidebarThumbnails *sidebar_thumbnails,
-			       int                  rotation)
-{
-	sidebar_thumbnails->priv->rotation = rotation;
-	if (sidebar_thumbnails->priv->loading_icons)
-		g_hash_table_remove_all (sidebar_thumbnails->priv->loading_icons);
-
-	if (sidebar_thumbnails->priv->document == NULL ||
-	    sidebar_thumbnails->priv->n_pages <= 0)
-		return;
-
-	ev_sidebar_thumbnails_clear_model (sidebar_thumbnails);
-	ev_sidebar_thumbnails_fill_model (sidebar_thumbnails);
-
-	/* Trigger a redraw */
-	sidebar_thumbnails->priv->start_page = -1;
-	sidebar_thumbnails->priv->end_page = -1;
-	g_idle_add ((GSourceFunc)refresh, sidebar_thumbnails);
-}
-
 static void
 ev_sidebar_tree_selection_changed (GtkTreeSelection *selection,
 				   EvSidebarThumbnails *ev_sidebar_thumbnails)
@@ -487,7 +601,7 @@ ev_sidebar_tree_selection_changed (GtkTreeSelection *selection,
 	page = gtk_tree_path_get_indices (path)[0];
 	gtk_tree_path_free (path);
 
-	ev_page_cache_set_current_page_history (priv->page_cache, page);
+	ev_document_model_set_page (priv->model, page);
 }
 
 static void
@@ -512,7 +626,7 @@ ev_sidebar_icon_selection_changed (GtkIconView         *icon_view,
 	gtk_tree_path_free (path);
 	g_list_free (selected);
 
-	ev_page_cache_set_current_page_history (priv->page_cache, page);
+	ev_document_model_set_page (priv->model, page);
 }
 
 static void
@@ -565,9 +679,8 @@ static gboolean
 ev_sidebar_thumbnails_use_icon_view (EvSidebarThumbnails *sidebar_thumbnails)
 {
 	EvSidebarThumbnailsPrivate *priv = sidebar_thumbnails->priv;
-	if (ev_page_cache_get_n_pages (priv->page_cache) > MAX_ICON_VIEW_PAGE_COUNT)
-		return FALSE;
-	return TRUE;
+
+	return (ev_document_get_n_pages (priv->document) <= MAX_ICON_VIEW_PAGE_COUNT);
 }
 
 static void
@@ -607,9 +720,8 @@ ev_sidebar_thumbnails_init (EvSidebarThumbnails *ev_sidebar_thumbnails)
 }
 
 static void
-page_changed_cb (EvPageCache         *page_cache,
-		 int                  page,
-		 EvSidebarThumbnails *sidebar)
+ev_sidebar_thumbnails_set_current_page (EvSidebarThumbnails *sidebar,
+					gint                 page)
 {
 	GtkTreeView *tree_view;
 	GtkTreePath *path;
@@ -638,6 +750,68 @@ page_changed_cb (EvPageCache         *page_cache,
 	gtk_tree_path_free (path);
 }
 
+static void
+page_changed_cb (EvSidebarThumbnails *sidebar,
+		 gint                 old_page,
+		 gint                 new_page)
+{
+	ev_sidebar_thumbnails_set_current_page (sidebar, new_page);
+}
+
+static gboolean
+refresh (EvSidebarThumbnails *sidebar_thumbnails)
+{
+	adjustment_changed_cb (sidebar_thumbnails);
+	return FALSE;
+}
+
+static void
+ev_sidebar_thumbnails_reload (EvSidebarThumbnails *sidebar_thumbnails)
+{
+	EvDocumentModel *model;
+
+	if (sidebar_thumbnails->priv->loading_icons)
+		g_hash_table_remove_all (sidebar_thumbnails->priv->loading_icons);
+
+	if (sidebar_thumbnails->priv->document == NULL ||
+	    sidebar_thumbnails->priv->n_pages <= 0)
+		return;
+
+	model = sidebar_thumbnails->priv->model;
+
+	ev_sidebar_thumbnails_clear_model (sidebar_thumbnails);
+	ev_sidebar_thumbnails_fill_model (sidebar_thumbnails);
+
+	/* Trigger a redraw */
+	sidebar_thumbnails->priv->start_page = -1;
+	sidebar_thumbnails->priv->end_page = -1;
+	ev_sidebar_thumbnails_set_current_page (sidebar_thumbnails,
+						ev_document_model_get_page (model));
+	g_idle_add ((GSourceFunc)refresh, sidebar_thumbnails);
+}
+
+static void
+ev_sidebar_thumbnails_rotation_changed_cb (EvDocumentModel     *model,
+					   GParamSpec          *pspec,
+					   EvSidebarThumbnails *sidebar_thumbnails)
+{
+	gint rotation = ev_document_model_get_rotation (model);
+
+	sidebar_thumbnails->priv->rotation = rotation;
+	ev_sidebar_thumbnails_reload (sidebar_thumbnails);
+}
+
+static void
+ev_sidebar_thumbnails_inverted_colors_changed_cb (EvDocumentModel     *model,
+						  GParamSpec          *pspec,
+						  EvSidebarThumbnails *sidebar_thumbnails)
+{
+	gboolean inverted_colors = ev_document_model_get_inverted_colors (model);
+
+	sidebar_thumbnails->priv->inverted_colors = inverted_colors;
+	ev_sidebar_thumbnails_reload (sidebar_thumbnails);
+}
+
 static void
 thumbnail_job_completed_callback (EvJobThumbnail      *job,
 				  EvSidebarThumbnails *sidebar_thumbnails)
@@ -646,6 +820,8 @@ thumbnail_job_completed_callback (EvJobThumbnail      *job,
 	GtkTreeIter *iter;
 
 	iter = (GtkTreeIter *) g_object_get_data (G_OBJECT (job), "tree_iter");
+	if (priv->inverted_colors)
+		ev_document_misc_invert_pixbuf (job->thumbnail);
 	gtk_list_store_set (priv->list_store,
 			    iter,
 			    COLUMN_PIXBUF, job->thumbnail,
@@ -655,18 +831,24 @@ thumbnail_job_completed_callback (EvJobThumbnail      *job,
 }
 
 static void
-ev_sidebar_thumbnails_set_document (EvSidebarPage	*sidebar_page,
-				    EvDocument          *document)
+ev_sidebar_thumbnails_document_changed_cb (EvDocumentModel     *model,
+					   GParamSpec          *pspec,
+					   EvSidebarThumbnails *sidebar_thumbnails)
 {
-	EvSidebarThumbnails *sidebar_thumbnails = EV_SIDEBAR_THUMBNAILS (sidebar_page);
-
+	EvDocument *document = ev_document_model_get_document (model);
 	EvSidebarThumbnailsPrivate *priv = sidebar_thumbnails->priv;
 
-	g_return_if_fail (EV_IS_DOCUMENT_THUMBNAILS (document));
+	if (!EV_IS_DOCUMENT_THUMBNAILS (document) ||
+	    ev_document_get_n_pages (document) <= 0 ||
+	    !ev_document_check_dimensions (document)) {
+		return;
+	}
 
-	priv->page_cache = ev_page_cache_get (document);
+	priv->size_cache = ev_thumbnails_size_cache_get (document);
 	priv->document = document;
-	priv->n_pages = ev_page_cache_get_n_pages (priv->page_cache);
+	priv->n_pages = ev_document_get_n_pages (document);
+	priv->rotation = ev_document_model_get_rotation (model);
+	priv->inverted_colors = ev_document_model_get_inverted_colors (model);
 	priv->loading_icons = g_hash_table_new_full (g_str_hash,
 						     g_str_equal,
 						     (GDestroyNotify)g_free,
@@ -701,15 +883,38 @@ ev_sidebar_thumbnails_set_document (EvSidebarPage	*sidebar_page,
 	}
 
 	/* Connect to the signal and trigger a fake callback */
-	g_signal_connect (priv->page_cache, "page-changed", G_CALLBACK (page_changed_cb), sidebar_thumbnails);
+	g_signal_connect_swapped (priv->model, "page-changed",
+				  G_CALLBACK (page_changed_cb),
+				  sidebar_thumbnails);
+	g_signal_connect (priv->model, "notify::rotation",
+			  G_CALLBACK (ev_sidebar_thumbnails_rotation_changed_cb),
+			  sidebar_thumbnails);
+	g_signal_connect (priv->model, "notify::inverted-colors",
+			  G_CALLBACK (ev_sidebar_thumbnails_inverted_colors_changed_cb),
+			  sidebar_thumbnails);
 	sidebar_thumbnails->priv->start_page = -1;
 	sidebar_thumbnails->priv->end_page = -1;
-	page_changed_cb (priv->page_cache,
-			 ev_page_cache_get_current_page (priv->page_cache),
-			 sidebar_thumbnails);
+	ev_sidebar_thumbnails_set_current_page (sidebar_thumbnails,
+						ev_document_model_get_page (model));
 	adjustment_changed_cb (sidebar_thumbnails);
 }
 
+static void
+ev_sidebar_thumbnails_set_model (EvSidebarPage   *sidebar_page,
+				 EvDocumentModel *model)
+{
+	EvSidebarThumbnails *sidebar_thumbnails = EV_SIDEBAR_THUMBNAILS (sidebar_page);
+	EvSidebarThumbnailsPrivate *priv = sidebar_thumbnails->priv;
+
+	if (priv->model == model)
+		return;
+
+	priv->model = model;
+	g_signal_connect (model, "notify::document",
+			  G_CALLBACK (ev_sidebar_thumbnails_document_changed_cb),
+			  sidebar_page);
+}
+
 static gboolean
 ev_sidebar_thumbnails_clear_job (GtkTreeModel *model,                                             
 			         GtkTreePath *path,
@@ -755,7 +960,6 @@ static void
 ev_sidebar_thumbnails_page_iface_init (EvSidebarPageIface *iface)
 {
 	iface->support_document = ev_sidebar_thumbnails_support_document;
-	iface->set_document = ev_sidebar_thumbnails_set_document;
+	iface->set_model = ev_sidebar_thumbnails_set_model;
 	iface->get_label = ev_sidebar_thumbnails_get_label;
 }
-
