@@ -143,6 +143,28 @@ launch_previewer (void)
 	return retval;
 }
 
+static gchar *
+get_label_from_filename (const gchar *filename)
+{
+	GFile   *file;
+	gchar   *label;
+	gboolean exists;
+
+	label = g_strrstr (filename, "#");
+	if (!label)
+		return NULL;
+
+	/* Filename contains a #, check
+	 * whether it's part of the path
+	 * or a label
+	 */
+	file = g_file_new_for_commandline_arg (filename);
+	exists = g_file_query_exists (file, NULL);
+	g_object_unref (file);
+
+	return exists ? NULL : label;
+}
+
 static void
 load_files (const char **files)
 {
@@ -174,7 +196,7 @@ load_files (const char **files)
 		const gchar *app_uri;
 
 		filename = files[i];
-		label = strchr (filename, '#');
+		label = get_label_from_filename (filename);
 		if (label) {
 			*label = 0;
 			label++;
@@ -242,7 +264,7 @@ main (int argc, char *argv[])
 
 #ifdef ENABLE_NLS
 	/* Initialize the i18n stuff */
-	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
+	bindtextdomain (GETTEXT_PACKAGE, ev_get_locale_dir());
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 #endif
